#ifndef DEBUG_H_
#define DEBUG_H_

#include <stdarg.h>
#include "globals.h"

typedef void (*printfch_t)(char c);

void chprintf(const char *fmt, ...);
void chsprintf(char *dest, const char *fmt, ...);
void chvsprintf(char *dest, const char *fmt, va_list va, printfch_t printfch);

#define printf chprintf
#define vprintf chvprintf

char *ltoa(char *p, long num, unsigned radix);

#ifdef LOG

void dbgInit(void);
void dbgSetChannelName(int channel, char *name);

//packet functions
void dbgStartS(int channel);
void dbgStartW(int channel);
void dbgStartR(int channel);
void dbgb(int channel, u8 b);
void dbgd(int channel, u8 *buffer, int length);
void dbgEnd(int channel);

//string functions
void dbgf(int channel, char *fmt, ...);
void dbgStr(int channel, char *string);

//data functions
void dbgDataR(int channel, void *buffer, int length);
void dbgDataW(int channel, void *buffer, int length);


#define SERIAL_DRIVER SerialDriver


inline static void serWriteByte(int channel, SERIAL_DRIVER *drv, u8 b)
{
	if(channel!=-1)
		dbgb(channel,b);
	sdPut(drv,b);
}
inline static void serWrite(int channel, SERIAL_DRIVER *drv, u8 *data, int length)
{
	if(channel!=-1)
		dbgd(channel,data,length);
	sdWrite(drv,data,length);
}
inline static int serReadByte(int channel, SERIAL_DRIVER *drv, int timeout)
{
	int r=sdGetTimeout(drv,MS2ST(timeout));
	if(drv->flags)
	{
		if(drv->flags&SD_PARITY_ERROR)
			dbgStr(channel,"SD_PARITY_ERROR");
		if(drv->flags&SD_FRAMING_ERROR)
			dbgStr(channel,"SD_FRAMING_ERROR");
		if(drv->flags&SD_OVERRUN_ERROR)
			dbgStr(channel,"SD_OVERRUN_ERROR");
		if(drv->flags&SD_NOISE_ERROR)
			dbgStr(channel,"SD_NOISE_ERROR");
		if(drv->flags&SD_BREAK_DETECTED)
			dbgStr(channel,"SD_BREAK_DETECTED");
		drv->flags=0;
	}


	if(r!=-1 && channel!=-1)
		dbgb(channel,r);
	return r;
}
inline static int serRead(int channel, SERIAL_DRIVER *drv, u8 *data, int length, int timeout)
{
	int r=sdReadTimeout(drv,data,length,MS2ST(timeout));
	if(r>0 && channel!=-1)
		dbgd(channel,data,r);
	return r;
}


#else

#define dbgInit() {}
#define dbgc(c) {}
#define dbgstr(string) {}
#define dbgstart(title) {}
#define dbg(fmt, ...) {}
#define dbgb(b) {}
#define dbgdata(buffer, length) {}

#endif

#endif /* DEBUG_H_ */
